package com.cmeeting.email;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.*;
import javax.mail.internet.*;
import java.io.File;
import java.util.Properties;
import java.util.concurrent.atomic.AtomicInteger;

@Slf4j
public class EmailSender {
    @Value("${email.sender}")
    private String SENDER;
    @Value("${email.sender.pwd}")
    private String EMAIL_PWD;
    @Value("${email.smtp.host}")
    private String SMTP_HOST;

    private static final Integer MAX_RETRY = 3;

    /**
     * @param toEmail 收件人
     * @param subject 邮件主题
     * @param attachmentPath 附件路径
     * @param recordFileId 转录文件ID
     * @return
     */
    public boolean sendEmailWithAttachment(String toEmail, String subject, String attachmentPath, String recordFileId) {
        // 邮件服务器配置
        Properties props = new Properties();
        props.put("mail.smtp.host", "smtp.office365.com");
        props.put("mail.smtp.auth", "true");

//        props.put("mail.smtp.port", "465");
//        props.put("mail.smtp.ssl.enable", "true"); // 使用SSL

        props.put("mail.smtp.port", "587");
        props.put("mail.smtp.starttls.enable", "true"); // 使用TLS

//        或者使用SSL
//        props.put("mail.smtp.port", "465");
//        props.put("mail.smtp.ssl.enable", "true");

        // 创建会话
        Session session = Session.getInstance(props,
                new javax.mail.Authenticator() {
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication("cmeeting_assistant@cimc.com", "scyou@xih45g6@xih4");
                    }
                });
        String body = "您好：\n" +
                "\n" +
                "         附件为您本次会议的会议纪要，烦请下载查看";

        AtomicInteger retryCount = new AtomicInteger(0);
        try {
            boolean isSent = false;
            while (retryCount.intValue() < MAX_RETRY && !isSent){
                // 创建邮件消息
                Message message = new MimeMessage(session);
                message.setFrom(new InternetAddress("cmeeting_assistant@cimc.com"));
                message.setRecipients(Message.RecipientType.TO,
                        InternetAddress.parse(toEmail));
                message.setSubject(subject);

                // 创建消息体部分（正文）
                MimeBodyPart messageBodyPart = new MimeBodyPart();
                messageBodyPart.setText(body);

                // 创建多部分消息
                Multipart multipart = new MimeMultipart();

                // 添加文本部分
                multipart.addBodyPart(messageBodyPart);

                // 添加附件部分
                if (attachmentPath != null && !attachmentPath.isEmpty()) {
                    MimeBodyPart attachmentPart = new MimeBodyPart();
                    DataSource source = new FileDataSource(attachmentPath);
                    attachmentPart.setDataHandler(new DataHandler(source));
                    attachmentPart.setFileName(new File(attachmentPath).getName());
                    multipart.addBodyPart(attachmentPart);
                }

                // 设置完整消息内容
                message.setContent(multipart);

                // 发送邮件
                Transport.send(message);
                log.error("邮件已成功发送: recordFileId->{}", recordFileId);
                isSent = true;
            }
        } catch (MessagingException e) {
            //todo 邮件失败记录
            // 异常处理
            retryCount.getAndIncrement();
            if (retryCount.intValue() > MAX_RETRY) {
                log.error("邮件发送达到最大重试次数: recordFileId->{}", recordFileId);
                throw new RuntimeException(e);
            }
            // 指数退避
            try {
                Thread.sleep((long) Math.pow(2, retryCount.intValue()) * 1000);
            } catch (InterruptedException ie) {
                Thread.currentThread().interrupt();
                throw new RuntimeException("邮件发送重试失败", ie);
            }
            return false;
        }
        return true;
    }
}