package com.cmeeting.job;

import com.cmeeting.service.FileProcessProducer;
import com.cmeeting.service.TecentMeetingService;
import com.cmeeting.service.WeComService;
import com.cmeeting.vo.TencentMeetingVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;

@Component
public class CmeetingJob {
    private static final Logger logger = LoggerFactory.getLogger(CmeetingJob.class);
    private static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    private static final Set<String> processedMeetingIds = Collections.synchronizedSet(new HashSet<>());
    @Autowired
    private WeComService weComService;
    @Autowired
    private TecentMeetingService tecentMeetingService;
    @Value(value = "${tencent.appId}")
    private String tencentAppId;
    @Value(value = "${tencent.sdkId}")
    private String tencentSdkId;
    @Value(value = "${tencent.secretId}")
    private String tencentSecretId;
    @Value(value = "${tencent.secretKey}")
    private String tencentSecretKey;
    @Value(value = "${tencent.admin.userId}")
    private String tencentAdminUserId;
    @Autowired
    private FileProcessProducer producer;

    /**
     * 企微人员定时同步
     */
//    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void weComUserSync() {
        try {
            logger.info("-------企微人员定时同步任务开始-------");
            logger.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));

            weComService.doUsers();
            logger.info("-------企微人员定时同步任务结束--------");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 企微人员定时同步
     */
//    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void TencentUserSync() {
        try {
            logger.info("-------腾讯会议人员定时同步任务开始-------");
            logger.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));

            tecentMeetingService.doUsers();
            logger.info("-------腾讯会议人员定时同步任务结束--------");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void execute() {
        // 定义时间格式化器
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        // 获取当前时间
        LocalDateTime now = LocalDateTime.now();
        // 往之前推两天
        LocalDateTime beforeDay = now.minusDays(2);

        // 转换为 Unix 时间戳（秒）并转为字符串
        Long nowTimestamp = now.atZone(ZoneId.systemDefault()).toEpochSecond();
        Long beforeDayTimestamp = beforeDay.atZone(ZoneId.systemDefault()).toEpochSecond();
        //日志记录
        logger.info("起始时间: " + beforeDay.format(formatter) + " | Unix 时间戳: " + beforeDayTimestamp);
        logger.info("结束时间: " + now.format(formatter) + " | Unix 时间戳: " + nowTimestamp);
        logger.info("----------------------------------");

//        dojob(beforeDayTimestamp, nowTimestamp);

        AtomicInteger currentPage = new AtomicInteger(1);
        TencentMeetingVO.TencentMeetingRequest request = TencentMeetingVO.TencentMeetingRequest.builder()
                .page(1)
                .pageSize(10)
                .startTime(beforeDayTimestamp)
                .endTime(nowTimestamp)
                .operatorId(tencentAdminUserId)
                .operatorIdType(1)
                .build();
        List<TencentMeetingVO.RecordFile> meetingFiles = tecentMeetingService.getMeetingFiles(request);


        if (meetingFiles == null || meetingFiles.isEmpty()) {
            logger.info("没有录制文件需要处理");
            return;
        }

        // 提交处理任务
        producer.submitBatchTasks(meetingFiles, "/");
    }
}
