package com.cmeeting.job;

import cn.chatbot.openai.completion.chat.ChatCompletionRequest;
import cn.chatbot.openai.completion.chat.ChatMessage;
import cn.chatbot.openai.completion.chat.ChatMessageRole;
import cn.chatbot.openai.completion.chat.Message;
import cn.chatbot.openai.service.LLMService;
import com.cmeeting.email.EmailSender;
import com.deepoove.poi.XWPFTemplate;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.tencentcloudapi.wemeet.Client;
import com.tencentcloudapi.wemeet.core.authenticator.AuthenticatorBuilder;
import com.tencentcloudapi.wemeet.core.authenticator.JWTAuthenticator;
import com.tencentcloudapi.wemeet.service.meetings.api.MeetingsApi;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdParticipantsGet200Response;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdParticipantsGet200ResponseParticipantsInner;
import com.tencentcloudapi.wemeet.service.records.api.RecordsApi;
import com.tencentcloudapi.wemeet.service.records.model.V1AddressesRecordFileIdGet200Response;
import com.tencentcloudapi.wemeet.service.records.model.V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xwpf.extractor.XWPFWordExtractor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;

import java.io.*;
import java.math.BigInteger;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.SecureRandom;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Data
@NoArgsConstructor
@Slf4j
public class FileProcessTask {
    private String recordFileId;
    private String meetingId;
    private String subMeetingId;
    private String savePath;
    private Map<String, Object> metadata;

    private int retryCount = 0;
    private static final int MAX_RETRY = 3;

    private String tencentAppId;
    private String tencentSdkId;
    private String tencentSecretId;
    private String tencentSecretKey;
    private String tencentAdminUserId;

    // 实际处理逻辑
    public void process() {
        try {
            boolean isSuccess = false;
            while (retryCount <= MAX_RETRY && !isSuccess) {
                Client client = new Client.Builder()
                        .withAppId(tencentAppId).withSdkId(tencentSdkId)
                        .withSecret(tencentSecretId,tencentSecretKey)
                        .build();

                // 获取参会成员明细
                MeetingsApi.ApiV1MeetingsMeetingIdParticipantsGetRequest participantsRequest =
                        new MeetingsApi.ApiV1MeetingsMeetingIdParticipantsGetRequest
                                .Builder(meetingId).subMeetingId(subMeetingId).operatorId(tencentAdminUserId).operatorIdType("1").build();
                AuthenticatorBuilder<JWTAuthenticator> participantsAuthenticatorBuilder =
                        new JWTAuthenticator.Builder()
                                .nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                                .timestamp(String.valueOf(System.currentTimeMillis() / 1000L));
                Map<String,Object> participantsMap = new ConcurrentHashMap<>();
                //主持人角色，以下角色都可以表示主持人
                //用户角色：
                //0：普通成员角色
                //1：创建者角色
                //2：主持人
                //3：创建者+主持人
                //4：游客
                //5：游客+主持人
                //6：联席主持人
                //7：创建者+联席主持人
                List<Long> hostRoleList = Arrays.asList(2L,3L,5L,6L,7L);
                try {
                    MeetingsApi.ApiV1MeetingsMeetingIdParticipantsGetResponse participantsResponse =
                            client.meetings().v1MeetingsMeetingIdParticipantsGet(participantsRequest, participantsAuthenticatorBuilder);
                    V1MeetingsMeetingIdParticipantsGet200Response data = participantsResponse.getData();
                    String scheduleStartTime = data.getScheduleStartTime();
                    String scheduleEndTime = data.getScheduleEndTime();
                    ZoneId zoneId = ZoneId.of("Asia/Shanghai");
                    DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                    String zonedDateTimeStart = Instant.ofEpochSecond(Long.valueOf(scheduleStartTime)).atZone(zoneId).format(df);
//                    String zonedDateTimeEnd = Instant.ofEpochSecond(Long.valueOf(scheduleEndTime)).atZone(zoneId).format(df);

                    participantsMap.put("meeting_date",zonedDateTimeStart);
                    participantsMap.put("meeting_location","线上腾讯会议");
                    List<V1MeetingsMeetingIdParticipantsGet200ResponseParticipantsInner> participants = data.getParticipants();
                    participantsMap.put("meeting_participants",
                            participants.stream()
                                    .map(item->new String(Base64.getDecoder().decode(item.getUserName()))).collect(Collectors.joining("、")));
                    Optional<V1MeetingsMeetingIdParticipantsGet200ResponseParticipantsInner> host = participants.stream().filter(item -> hostRoleList.contains(item.getUserRole())).findFirst();
                    participantsMap.put("meeting_host",host.isPresent() ? new String(Base64.getDecoder().decode(host.get().getUserName())) : "未知主持人");
                } catch (Exception e) {
                    e.printStackTrace();
                    log.error(e.getMessage());
                    throw new RuntimeException(e);
                }



                //查询录制转写详情
                RecordsApi.ApiV1AddressesRecordFileIdGetRequest addressRequest =
                        new RecordsApi.ApiV1AddressesRecordFileIdGetRequest.Builder(recordFileId)
                                .operatorId(tencentAdminUserId)
                                .operatorIdType("1")
                                .build();
                RecordsApi.ApiV1AddressesRecordFileIdGetResponse addressResponse =
                        client.records().v1AddressesRecordFileIdGet(addressRequest,
                                new JWTAuthenticator.Builder().nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                                        .timestamp(String.valueOf(System.currentTimeMillis() / 1000L)));
                // 处理响应
                if (addressResponse != null && addressResponse.getData() != null) {
                    log.info("Successfully got address for record file {}", recordFileId);
                    V1AddressesRecordFileIdGet200Response addressData = addressResponse.getData();

                    // 获取AI会议转录文件
                    List<V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner> transcripts =
                            addressData.getAiMeetingTranscripts();

                    if (transcripts != null && !transcripts.isEmpty()) {
                        log.info("Found {} AI meeting transcripts for record file {}",
                                transcripts.size(), recordFileId);

                        // 处理每个转录文件
                        for (V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner transcript : transcripts) {
                            String fileType = transcript.getFileType();
                            String downloadUrl = transcript.getDownloadAddress();

                            if ("txt".equalsIgnoreCase(fileType)) {
                                log.info("AI Transcript - Type: {}, URL: {}", fileType, downloadUrl);

                                // 1. 下载文件
                                byte[] fileData = downloadFile(downloadUrl);

                                // 2. 将二进制文件转换为文本
                                String recordTextContent = new String(fileData);
//                                String recordTextContent = new String(Files.readAllBytes(Paths.get("/20250321145249-天达物流AIGC维修助手需要沟通-转写原文版-1.txt")));
                                if(StringUtils.isEmpty(recordTextContent.replaceAll("\\n","").trim())){
                                    log.info("获取的转录文本为空，跳过纪要生成，URL:{}，fileRecordId:{}", fileType, downloadUrl,recordFileId);
                                    continue;
                                }

                                // 3. 处理文件 (调用Claude API等)
                                String processedResult = processWithClaude(recordTextContent);

                                // 4. 保存结果
                                saveResult(savePath, processedResult, participantsMap);
                            }
                        }
                    } else {
                        log.info("No AI meeting transcripts found for record file {}", recordFileId);
                    }

                } else {
                    log.warn("Empty response for record file: {}", recordFileId);
                }
                isSuccess = true;
            }
        } catch (Exception e) {
            // 异常处理
            retryCount++;
            if (retryCount > MAX_RETRY) {
                log.error("达到最大重试次数: {}", recordFileId);
                throw new RuntimeException(e);
            }
            // 指数退避
            try {
                Thread.sleep((long) Math.pow(2, retryCount) * 1000);
            } catch (InterruptedException ie) {
                Thread.currentThread().interrupt();
                throw new RuntimeException("重试失败", ie);
            }
        }
    }
    
    private byte[] downloadFile(String url) {
        // 实现文件下载逻辑
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder().url(url).build();
        try {
            Response response = client.newCall(request).execute();
            if (!response.isSuccessful()) throw new IOException("Unexpected code " + response);
            return response.body().bytes();
        }catch (Exception e){
            throw new RuntimeException("下载文件失败", e);
        }
    }

    private String getRecordTextContent(byte[] fileData) {
        // 定义文件路径和文件名
        String fullPath = savePath + (System.currentTimeMillis() / 1000L) +  ".docx";
        // 将字节内容写入本地文件
        try (FileOutputStream fos = new FileOutputStream(fullPath)) {
            fos.write(fileData);
            log.info("DOCX transcript saved to: {}", fullPath);
        } catch (IOException e) {
            log.error("Error saving DOCX transcript to file: {}", e.getMessage(), e);
        }
        XWPFDocument document;
        try {
            FileInputStream fis = new FileInputStream(fullPath);
            document = new XWPFDocument(fis);
        } catch (IOException e) {
            throw new RuntimeException(e.getMessage());
        }
        XWPFWordExtractor extractor = new XWPFWordExtractor(document);
        String textContent = extractor.getText();
        log.info("DOCX content as string:\n{}", textContent);
        return textContent;
    }
    
    private String processWithClaude(String textContent) {
        //将文件传送给大模型处理
        String token = "AKIAXFAXF62IWJXGLVEE.LnKInaahcMZG9zLsGMH3nTLOw3S3lK5Vcu0+ifnO";
        String apiAddr = "https://bedrock.chatbot.cn/llm/sse-invoke";
        String model = "anthropic.claude-3-5-sonnet-20240620-v1:0";
        int maxTokens = 5000;

        List<Message> messages = new ArrayList<>();
        ChatMessage chatMessage = new ChatMessage(ChatMessageRole.USER.value(), "你是会议纪要助手，基于提供的会议记录，生成一份详细的会议纪要，并严格按照指定的XML格式输出。\\n\\n要求：\\n1. 会议名称：应简洁明了地反映会议主题，不超过30个字\\n2. 会议目的：需清晰阐述召开此次会议的原因和期望达成的目标，不超过50字\\n3. 会议要点：需全面概括会议内容，让读者理解会议讨论的核心问题，不要遗漏任何有价值的信息，每个要点需用序号标注并另起一行\\n4. 会议内容：需详细记录会议的背景信息、主要讨论内容、各参会人员的发言要点、达成的共识和决策等。内容应有逻辑性，分段清晰，重点突出，不可遗漏关键信息\\n5. 会议沟通内容：需按照一定的顺序组织内容，详细描述每个事项的内容，不要使用流水账形式，应包含具体的沟通过程、各方观点和达成的一致意见\\n6. 会后跟进事项：详细描述会议结束之后，每个后续事项的责任人、事项内容、完成时间（若会议记录中没有提及，则填写待定）等信息，事项要具体到责任人\\n\\n注意事项：\\n1. 所有内容需基于原始会议记录，不要添加虚构信息\\n2. 会议纪要内容需详细充分，建议总字数不少于3000字，实际字数可根据原始会议记录的丰富程度进行适当调整\\n\\n输出格式必须严格按照以下XML结构，不允许有任何嵌套标签：\\n\\n<meeting_name>会议名称</meeting_name>\\n<meeting_purpose>会议目的</meeting_purpose>\\n<meeting_key_points>会议要点</meeting_key_points>\\n<meeting_content>会议内容</meeting_content>\\n<meeting_communication>会议沟通内容</meeting_communication>\\n<meeting_follow_up>\\n在此填写会议跟踪事项，格式为：\\n1. 内容：XXX；责任人：XXX；完成时间：XXXX-XX-XX\\n2. 内容：XXX；责任人：XXX；完成时间：XXXX-XX-XX\\n...\\n</meeting_follow_up>\\n\\n完成XML格式的会议纪要后，必须在XML之外单独添加猜您想问部分，生成3-4条与会议内容相关的推荐问题。\\n\\n会议记录如下：\\n");
        messages.add(chatMessage);
        chatMessage = new ChatMessage(ChatMessageRole.ASSISTANT.value(), "好的请提供会议记录");
        messages.add(chatMessage);
        chatMessage = new ChatMessage(ChatMessageRole.USER.value(), textContent);

        messages.add(chatMessage);

        // 调用Claude API处理文件
        String ret = call_llm(apiAddr, model, token, messages, maxTokens);
        return ret;
    }

    private void saveResult(String path, String content, Map<String,Object> participantsMap) {
        String nowTime = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd-HHmmss"));
        String targetPath = path + nowTime + ".md";  // 改为.md扩展名;
        String targetFileName;
        String meetingName;
        // 保存处理结果
        try {
            // 将字符串写入Markdown文件
            Files.write(Paths.get(targetPath), content.getBytes());
            log.info("Markdown文件已保存到: {}", targetPath);

//            convert(targetPath,targetWordPath);
            String markdownContent = new String(Files.readAllBytes(Paths.get(targetPath)));
            Map<String, Object> dataModel = convertXmlToMap(markdownContent);
            dataModel.putAll(participantsMap);

            XWPFTemplate template = XWPFTemplate.compile("/data_net_template.docx").render(dataModel);
            meetingName = dataModel.get("meeting_name") != null ? String.valueOf(dataModel.get("meeting_name")) : "腾讯会议纪要";
            targetFileName = meetingName + "_" + nowTime;
            template.writeAndClose(new FileOutputStream(path + targetFileName + ".docx"));
        } catch (Exception e) {
            log.error("填充会议纪要失败: {}", e.getMessage(), e);
            throw new RuntimeException("填充会议纪要失败");
        }

        //邮件推送
        EmailSender emailSender = new EmailSender();
        emailSender.sendEmailWithAttachment("duanxincheng@chatbot.cn",meetingName,path + targetFileName + ".docx",recordFileId);
//        emailSender.sendEmailWithAttachment("xuwentao@chatbot.cn",meetingName,path + targetFileName + ".docx",recordFileId);
//        emailSender.sendEmailWithAttachment("jiaqi.cai@cimc.com",meetingName,path + targetFileName + ".docx",recordFileId);
    }

    public static String call_llm(String apiAddr, String model, String token, List<Message> messages, int maxTokens) {
        LLMService service = new LLMService(token, apiAddr);
        StringBuilder stringBuilder = new StringBuilder();
        try {
            ChatCompletionRequest chatCompletionRequest = ChatCompletionRequest.builder()
                    .model(model)
                    .messages(messages)
                    .stream(true)
                    .n(1)
                    .maxTokens(maxTokens)
                    .logitBias(new HashMap<>())
                    .build();

            service.streamChatCompletion(chatCompletionRequest).doOnError(Throwable::printStackTrace).blockingForEach(chunk -> {
                chunk.getChoices().stream().map(choice -> choice.getMessage().getContent())
                        .filter(Objects::nonNull).findFirst().ifPresent(o -> {
                    try {
                        stringBuilder.append(new String(o.getBytes(Charset.defaultCharset())));
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                });
            });
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        service.shutdownExecutor();
        return stringBuilder.toString();
    }
    public static Map<String, Object> convertXmlToMap(String markdownContent) throws Exception {
        String xmlContent = extractXmlFromMarkdown(markdownContent);

        XmlMapper xmlMapper = new XmlMapper();
        ObjectMapper objectMapper = new ObjectMapper();

        // 先将 XML 读取为 Map
        Map<?, ?> xmlMap = xmlMapper.readValue(xmlContent, Map.class);

        // 转换为更标准的 Map<String, Object>
        return objectMapper.convertValue(xmlMap, Map.class);
    }

    /**
     * markdown转xml
     * @param markdown
     * @return
     */
    private static String extractXmlFromMarkdown(String markdown) {
        StringBuffer sb = null;
        try {
            int start = markdown.indexOf("<");
            int end = markdown.lastIndexOf(">") + 1;
            sb = new StringBuffer();
            sb.append("<root>");
            sb.append(markdown.substring(start, end).trim());
            sb.append("</root>");
        } catch (Exception e) {
            log.info("markdown转xml，markdown->{}",markdown);
            throw new RuntimeException(e.getMessage());
        }
        return sb.toString();
    }


    public FileProcessTask(String recordFileId, String meetingId, String subMeetingId, String savePath, Map<String, Object> metadata, String tencentAppId,
                           String tencentSdkId, String tencentSecretId, String tencentSecretKey, String tencentAdminUserId) {
        this.recordFileId = recordFileId;
        this.savePath = savePath;
        this.metadata = metadata;
        this.tencentAppId = tencentAppId;
        this.tencentSdkId = tencentSdkId;
        this.tencentSecretId = tencentSecretId;
        this.tencentSecretKey = tencentSecretKey;
        this.tencentAdminUserId = tencentAdminUserId;
        this.meetingId = meetingId;
        this.subMeetingId = subMeetingId;
    }
}