package com.cmeeting.service;

import com.cmeeting.job.FileProcessTask;
import com.cmeeting.vo.TencentMeetingVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

@Service
@Slf4j
public class FileProcessProducer {
    
    @Autowired
    private ThreadPoolTaskExecutor fileProcessExecutor;
    
    @Autowired
    private FileProcessCallbackHandler callbackHandler;
    @Value(value = "${tencent.appId}")
    private String tencentAppId;
    @Value(value = "${tencent.sdkId}")
    private String tencentSdkId;
    @Value(value = "${tencent.secretId}")
    private String tencentSecretId;
    @Value(value = "${tencent.secretKey}")
    private String tencentSecretKey;
    @Value(value = "${tencent.admin.userId}")
    private String tencentAdminUserId;
    
    // 批量提交任务
    public void submitBatchTasks(List<TencentMeetingVO.RecordFile> recordFiles, String baseSavePath) {
        List<Future<?>> futures = new ArrayList<>();
        
        for (TencentMeetingVO.RecordFile recordFile : recordFiles) {
            // 为每个URL创建任务
            FileProcessTask task = new FileProcessTask(
                recordFile.getRecordFileId(),
                recordFile.getMeetingId(),
                recordFile.getSubMeetingId(),
                "/save/",
                Collections.emptyMap(),
                tencentAppId,
                tencentSdkId,
                tencentSecretId,
                tencentSecretKey,
                tencentAdminUserId
            );
            
            // 提交任务到线程池
            Future<?> future = fileProcessExecutor.submit(() -> {
                task.process();
                callbackHandler.onComplete(task); // 回调处理
            });
            
            futures.add(future);
        }
        
        // 可以添加一个监控线程来检查所有任务完成情况
        monitorTaskCompletion(futures);
    }
    
    private void monitorTaskCompletion(List<Future<?>> futures) {
        new Thread(() -> {
            for (Future<?> future : futures) {
                try {
                    future.get(); // 阻塞直到任务完成
                } catch (InterruptedException | ExecutionException e) {
                    log.error("任务执行异常", e);
                }
            }
            callbackHandler.onAllComplete(); // 所有任务完成回调
        }).start();
    }
}