package com.cmeeting.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.mapper.primary.WeComUserMapper;
import com.cmeeting.pojo.WeComUser;
import com.cmeeting.service.WeComService;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Collection;
import java.util.List;

import static com.cmeeting.WeComAndTencentMeeting.convertJsonToWeComUsers;
import static com.cmeeting.WeComAndTencentMeeting.markDuplicateNames;

@Service
public class WeComServiceImpl extends ServiceImpl<WeComUserMapper, WeComUser>  implements WeComService {

    private static final String BASE_URL = "https://qyapi.weixin.qq.com/cgi-bin/user/simplelist";

    private static final String ACCESS_TOKEN = "z8g0fkzlVBMKrwi67CRwE7gcnGqjOMrTK6LDih06vs1ixpV9BkrVWQJkoF3HFf5a2UrMvTKEBewOo7wPGvVxNjV_ZGOp_dUxTfnCG6kRUK9LKXPlhskGOI3mcI3ZZpGqpPcgjCsE877hh1BYTB_i3t5X_q1tZfv0K636o3a0HyAMf4GlzviCd1vQHfkGf3p15FjnATwDV8ddhMZo8U-_uA";

    private static final String CORP_ID = "wx34544d057db97ffd";
    private static final String CORP_SECRET = "7YLePWG7rJqkQFnAB4FeylqAXpmu7q5qv_NOeSGNbm0";

    @Resource
    private WeComUserMapper weComUserMapper;

    @Override
    public void batchInsert(List<WeComUser> users) {
        weComUserMapper.batchInsert(users);
    }

    @Override
    public void doUsers() throws Exception {
        // 示例：获取部门ID为6的用户列表
        int departmentId = 6;
        JsonObject departmentList = getDepartmentList();
        JsonObject result = getUserListByDepartment(departmentId);
        // 处理返回结果
        if (result.get("errcode").getAsInt() == 0) {
            System.out.println("企微获取用户列表成功：");
            JsonArray userList = result.getAsJsonArray("userlist");
            for (int i = 0; i < userList.size(); i++) {
                JsonObject user = userList.get(i).getAsJsonObject();
                System.out.println("姓名: " + user.get("name").getAsString() +
                        ", UserID: " + user.get("userid").getAsString());
            }
            // 2. 转换为WeComUser集合
            List<WeComUser> users = convertJsonToWeComUsers(result);
            // 3. 检查重名并设置标志
            markDuplicateNames(users);

            // 4. 批量插入数据库
            batchInsert(users);
        }
    }

    @Override
    public String getToken() {
        //获取token
//        String url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=ww1fd8778458e9f1e8&corpsecret=uFRq9Xi8-dVY90LydXYBhjc91JnnfkPUR6lHDdeJ_fo";
        String url = "https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid=" + CORP_ID + "&corpsecret=" + CORP_SECRET;
        String accessToken = "";
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();

        try (Response response = client.newCall(request).execute()) {
            if (response.isSuccessful()) {
                String responseBody = response.body().string();
                JSONObject jsonResponse = new JSONObject(responseBody);
                accessToken = jsonResponse.getString("access_token");
            } else {
                throw new RuntimeException("Failed to fetch token. HTTP Code: " + response.code());
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return accessToken;
    }

    /**
     * 获取企微部门列表
     *
     * @return 包含用户列表的JsonObject
     * @throws Exception
     */
    public JsonObject getDepartmentList() throws Exception {
        String urlStr = "https://qyapi.weixin.qq.com/cgi-bin/department/simplelist?access_token=" + getToken();

        URL url = new URL(urlStr);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");

        int responseCode = conn.getResponseCode();
        if (responseCode == HttpURLConnection.HTTP_OK) {
            BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String inputLine;
            StringBuilder response = new StringBuilder();

            while ((inputLine = in.readLine()) != null) {
                response.append(inputLine);
            }
            in.close();

            // 使用Gson解析JSON响应
            Gson gson = new Gson();
            return gson.fromJson(response.toString(), JsonObject.class);
        } else {
            throw new RuntimeException("HTTP GET请求失败，错误码: " + responseCode);
        }
    }

    /**
     * 根据企业微信部门ID获取用户列表
     *
     * @param departmentId 部门ID
     * @return 包含用户列表的JsonObject
     * @throws Exception
     */
    public JsonObject getUserListByDepartment ( int departmentId) throws Exception {
        String urlStr = BASE_URL + "?access_token=" + ACCESS_TOKEN + "&department_id=" + departmentId;

        URL url = new URL(urlStr);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");

        int responseCode = conn.getResponseCode();
        if (responseCode == HttpURLConnection.HTTP_OK) {
            BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String inputLine;
            StringBuilder response = new StringBuilder();

            while ((inputLine = in.readLine()) != null) {
                response.append(inputLine);
            }
            in.close();

            // 使用Gson解析JSON响应
            Gson gson = new Gson();
            return gson.fromJson(response.toString(), JsonObject.class);
        } else {
            throw new RuntimeException("HTTP GET请求失败，错误码: " + responseCode);
        }
    }
}
